#employee-database.choinski.net

The [employee-database.choinski.net](http://employee-database.choinski.net/) application demonstrates a PHP MVC pattern developed for small web projects.  This application requires [PHP](http://www.php.net/) version 5.3 (or greater) and [SQLite3](http://www.sqlite.org/).


## PHP MVC Pattern

It is important to note that this application demonstrates a _pattern_ and not a _framework_.

The pattern consists of the following elements:

 * Routing
 * Controller
 * Model
 * View

### Routing

The _routing_ of requests is handled by the web server (through a configuration file, such as a _.htaccess_ or _VirtualHost_ file).  The routing rules can be as basic as the following example:

	RewriteEngine On
	
	RewriteRule ^([\w]+)/$ 						/path/to/controller/$1.php
	RewriteRule ^([\w]+)/([\w]+)/$ 				/path/to/controller/$1.php?action=$2
	RewriteRule ^([\w])/([\w])/([0-9]{1,10}/$	/path/to/controller/$1.php?action=$2&id=$3
	
In this routing rules example, the first matching group (`([\w]+)`) refers to the _Model_; the second matching group (`([\w]+)`) refers to the _Action_ to be performed on the _Model_; and the last matching group (`([0-9]{1,10}`) refers to an _Id_ of a _Model_ record.  

For example, a URL of `http://myapp.com/person/show/1/` is routed to the _Person Controller_, which calls the _Show Action_ to display a _View_ containing a _Person_ record with an _Id_ of "1". 

### Controller

The _Controller_ processes the routing request and displays a _View_ containing a _Model_ object (or objects).  The following is an example of a _Controller_:

	<?php
	
	namespace MattChoinski\MyApp\Controller
	{
			require_once( realpath( $_SERVER[ "DOCUMENT_ROOT"] ) . "/controller/base.php" );
				
			class Person extends Base
			{
				public function Edit()
				{
					$person = \MattChoinski\MyApp\Model\Person::FetchById();
					
					require_once( DOCUMENT_ROOT . "/view/person/form.php" );
				}
				
				public function List()
				{
					$personCollection = \MattChoinski\MyApp\Model\Person::FetchAll();
					
					require_once( DOCUMENT_ROOT . "/view/person/list.php" );
				}
				
				public function Set()
				{
					\MattChoinski\MyApp\Model\Person::Set();
					
					header( "Location: /person/" );
				}
				
				public function Show()
				{
					$person = \MattChoinski\MyApp\Model\Person::FetchById();
					
					require_once( DOCUMENT_ROOT . "/view/person/show.php" );
				}
			}
	}
	
	namespace
	{
		$action = (
			isset( $_GET[ "action" ] )
				? $_GET[ "action" ]
				: "List" );
		$controller = new MattChoinski\MyApp\Controller\Person();
		
		call_user_func( array( $controller, $action ) );
	}
	
This example _Controller_ contains a few _Actions_ ("Edit", "List", "Set" and "Show).  (This example _Controller_ extends a [_Base Controller_](https://raw.github.com/mattchoinski/employee-database.choinski.net/master/controller/base.php), which simply provides common functionality to all _Controllers_.)

It is important to note that there is a "default" _namespace_ that instantiates the _Controller_ - with the appropriate _Action_ - through the `call_user_func` function.

The requested (or defaulted) _Action_ then calls the appropriate _Model_ (or _Models_) and passes the data (or data collections) to a _View_.

### Model

The _Model_ simply encapsulates the business logic and data management, as shown in the following example:

	<?php

	namespace MattChoinski\MyApp\Model
	{
		require_once( realpath( $_SERVER[ "DOCUMENT_ROOT" ] ) . "/model/base.php" );
	
		class Person extends Base
		{
			private static $dataQuery = <<<DataQuery
	SELECT Id
	      ,DateCreated
	      ,Email
	      ,NameFirst
	      ,NameLast
	  FROM Person
	DataQuery;
   
			public $Email;
			public $NameFirst;
			public $NameLast;
		
		
			public static function FetchAll( )
			{
				return self::Fetch( self::$dataQuery ) );
			}

			public static function FetchById( 
				$id = null )
			{
				$result = null;
			
				if ( !isset( $id ) )
				{
					$id = $_GET[ "id" ];
				}
				$dataCriteria = <<<DataCriteria
	 WHERE a.Id = $id;
	DataCriteria;
				$modelCollection =
					self::Fetch(
						self::$dataQuery 
						. $dataCriteria ) );
				if ( !empty( $modelCollection[ 0 ] ) )
				{
					$result = $modelCollection[ 0 ];
				}

				return $result;
			}
		
			private static function Insert(
				$model )
			{		
				$dataQuery = <<<DataQuery
	INSERT 
	  INTO Employee
	       (
	         Email
	        ,NameFirst
	        ,NameLast
	       )
	VALUES
	       (
	         '{$model->Email}'
	        ,'{$model->NameFirst}'
	        ,'{$model->NameLast}'
	       );
	DataQuery;

				return parent::Execute( $dataQuery );
			}
		
			public static function Set()
			{
				$result = 0;
			
				$model = new Person();
	 			$model->Id = 
	 				filter_input( 
	 					INPUT_POST, 
	 					"id", 
	 					FILTER_SANITIZE_NUMBER_INT );
	 			$model->Email =
	 				filter_input(
	 					INPUT_POST,
	 					"email",
	 					FILTER_SANITIZE_STRING );
	 			$model->NameFirst =
	 				filter_input(
	 					INPUT_POST,
	 					"name_first",
	 					FILTER_SANITIZE_STRING );
	 			$model->NameLast =
	 				filter_input(
	 					INPUT_POST,
	 					"name_last",
	 					FILTER_SANITIZE_STRING );
				if ( $model->Id == 0 )
				{
					$result = self::Insert( $model );
				}
				else
				{
					self::Update( $model );
					$result = $model->Id;
				}
				
				return $result;
			}
		
			private static function Update(
				$model )
			{
				$dataQuery = <<<DataQuery
	UPDATE Person
	   SET CreatedById = {$_SESSION[ "id" ]}
	      ,Email = '{$model->Email}'
	      ,NameFirst = '{$model->NameFirst}'
	      ,NameLast = '{$model->NameLast}'
	 WHERE Id = {$model->Id};
	DataQuery;

				parent::Execute( $dataQuery );
			}
		}
	}
	
This example _Model_ extends a [_Base Model_](https://raw.github.com/mattchoinski/employee-database.choinski.net/master/model/base.php), which simply provides common functionality, such as data binding to objects, to all _Models_.

### View   

The _View_ simply handles the display of an object (or a collection of objects), as shown in the following example:

	<?php

	require_once( DOCUMENT_ROOT . "/include/script/header.php" );
	
	?>
			<section>
				<h1>All Employees</h1>
				<br /><br />
	<?php

	if ( !empty( $peopleCollection ) )
	{
		foreach( $peopleCollection as $person )
		{
			echo <<<Html
				<a href="/person/edit/{$person->Id}/">{$person->NameLast}, {$person->NameFirst}</a><br />
	Html;
		}
	}
	
	?>
			</section>
	<?php

	require_once( DOCUMENT_ROOT . "/include/script/footer.php" );


It is important to note that the _View_, as shown in the above example, does not directly interact with the _Model_.  Instead, the _View_ receives _Model_ data from the _Controller_.
   
--- 
Copyright &copy; 2012 Matthew Choinski, http://mattchoinski.com

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.  If not, see [http://www.gnu.org/licenses/](http://www.gnu.org/licenses/).
 